﻿namespace CnBlogsDemos.ReportingSyncer
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;


    public class CommandLine
    {
        protected Dictionary<string, string> _command;
        protected string _defaultConfig;

        public string InputDir
        {
            get
            {
                if (_command.ContainsKey("input"))
                {
                    return _command["input"];
                }
                throw new Exception("The expected parameter /input:[directory] is missing!");
            }
        }

        public string ConfigFilePath
        {
            get
            {
                if (_command.Keys.Contains("config"))
                {
                    return _command["config"];
                }
                return _defaultConfig;
            }
        }

        public string TaskID
        {
            get
            {
                if (_command.ContainsKey("taskid"))
                {
                    return _command["taskid"];
                }
                throw new Exception("The expected parameter /taskid:[id] is missing!");
            }

        }

        public string TaskName
        {
            get
            {
                if (_command.ContainsKey("taskname"))
                {
                    return _command["taskname"];
                }
                throw new Exception("The expected parameter /taskname:[name] is missing!");
            }

        }

        public CommandLine(string[] args)
        {
            ParseCommandLine(args);
         
        }

        public virtual string PrintInputParameters()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("Input Parameter:");
            foreach (KeyValuePair<string, string> param in _command)
            {
                sb.Append(string.Format("/{0}:{1}\t", param.Key, param.Value));
            }
            return sb.ToString();
        }

        private void ParseCommandLine(string[] args)
        {
            if (0 == args.Length ||
                (1 == args.Length &&
                (args[0].Contains("?") || args[0].ToLower().Contains("help"))))
            {
                Prompt();
                throw new ArgumentException("");
            }

            _command = new Dictionary<string, string>();
            try
            {
                foreach (string arg in args)
                {
                    string[] param = arg.ToLower().Split(new char[] { ':' }, 2);
                    _command.Add(param[0].Replace("/", ""), param[1]);
                }
            }
            catch (Exception e)
            {
                throw new ArgumentException(e.Message);
            }
        }

        protected virtual void CheckMandatoryParameter()
        {
            throw new NotImplementedException();
        }

        protected virtual void Prompt()
        {
            throw new NotImplementedException();
        }
    }
}