﻿namespace CnBlogsDemos.ReportingSyncer
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.IO;
    using System.Data;
    using CnBlogsDemos.ReportingDBManager;


    public class DataSyncer
    {
        private CommandLine _cmd;

        public DataSyncer(CommandLine cmd)
        {
            // TODO: Complete member initialization
            this._cmd = cmd;
        }

        internal int RunInCommandLineMode()
        {
            int iRet = -1;
            int taskId = 0;
            string inputDir = _cmd.InputDir;
            string taskName = string.Empty;
            string type = string.Empty;

            //Get taskid
            int.TryParse(_cmd.TaskID, out taskId);
            // if after parsing ,task id still equals 0 ,return
            if (taskId.Equals(0))
            {
                Console.WriteLine("taskid is invalid");
                return -1;
            }
            // if input directory is not exist parsing return
            if (!Directory.Exists(inputDir))
            {
                Console.WriteLine("input directory is not exist");
                return -1;
            }
            else
            {
                //read files from input dir
                var files = Directory.GetFiles(inputDir);

                if (!files.Any())
                {
                    Console.WriteLine("there is not files under input directory");
                    return -1;
                }
                else
                {
                    foreach (var file in files)
                    {
                        if (file.Contains("trend.txt"))
                        {
                            type = "trend";
                            InsertAnalysisData(file, taskId, taskName, type);
                        }
                        else if (file.Contains("bar.txt"))
                        {
                            type = "bar";
                            InsertAnalysisData(file, taskId, taskName, type);
                        }
                    }
                }


            }



            return iRet;

        }

        public void InsertAnalysisData(string filePath, int taskId, string taskName, string type)
        {
            if (File.Exists(filePath))
            {

                if (type.Equals("bar", StringComparison.OrdinalIgnoreCase))
                {
                    var dataRows = this.GetTableFromCSV(filePath).AsEnumerable();
                    // 创建一个新的 dbstore context 实例
                    using (DbStoreContext dsc = new DbStoreContext())
                    {
                        // 如果 Bars表 存在任何相同的TaskID，删除相关记录
                        if (dsc.Bars.Any(n => n.TaskID.Equals(taskId)))
                        {
                            foreach (var item in dsc.Bars.Where(n => n.TaskID.Equals(taskId)))
                            {
                                dsc.Bars.Remove(item);
                            }
                        }
                        
                        foreach (var row in dataRows)
                        {
                            // 注意 ，这个 row.Field<string>(index);中的index 对应我们csv/txt 文件中的数据列，索引从0开始
                            Bar b = new Bar();
                            b.KeyWord = row.Field<string>(0);
                            b.B1Better = Convert.ToInt32(row.Field<string>(1));
                            b.Equal = Convert.ToInt32(row.Field<string>(2));
                            b.B2Better = Convert.ToInt32(row.Field<string>(3));
                            b.Winner = row.Field<string>(4);
                            b.IsActive = true;
                            b.type = type;
                            b.TaskID = taskId;
                            b.TaskName = taskName;
                            dsc.Bars.Add(b);
                        }
                        //保存修改。EF会去执行插入数据到DB，默认开始事务
                        dsc.SaveChanges();
                    }
                }
                else if (type.Equals("trend", StringComparison.OrdinalIgnoreCase))
                {
                    var tailWholePageTrendDataRows = this.GetTableFromCSV(filePath).AsEnumerable();
                    using (DbStoreContext dsc = new DbStoreContext())
                    {
                        if (dsc.Trends.Any(n => n.TaskID.Equals(taskId)))
                        {
                            foreach (var item in dsc.Trends.Where(n => n.TaskID.Equals(taskId)))
                            {
                                dsc.Trends.Remove(item);
                            }
                        }
                        foreach (var row in tailWholePageTrendDataRows)
                        {
                            // 注意 ，这个 row.Field<string>(index);中的index 对应我们csv/txt 文件中的数据列，索引从0开始
                            Trend t = new Trend();
                            t.id = Convert.ToInt32(row.Field<string>(0));
                            t.TaskID = taskId;
                            t.Time = Convert.ToDateTime(row.Field<string>(3));
                            t.B1Better = Convert.ToInt32(row.Field<string>(4));
                            t.Equal = Convert.ToInt32(row.Field<string>(5));
                            t.B2Better = Convert.ToInt32(row.Field<string>(6));
                            t.UnCertain = Convert.ToInt32(row.Field<string>(7));
                            t.GrandTotal = Convert.ToInt32(row.Field<string>(8));
                            t.TaskName = taskName;
                            t.IsActive = true;
                            t.type = type;
                            dsc.Trends.Add(t);
                        }
                        dsc.SaveChanges();
                    }

                }

            }
        }

        /// <summary>
        /// GetTableFromCSV file
        /// </summary>
        /// <param name="csvFilePath">file path</param>
        /// <returns>data table</returns>
        private  DataTable GetTableFromCSV(string csvFilePath)
        {
            using (StreamReader sr = new StreamReader(csvFilePath))
            {
                string line = sr.ReadLine();
                if (!string.IsNullOrEmpty(line))
                {
                    string[] columns = line.Split('\t');
                    DataTable dt = new DataTable();
                    foreach (string columnName in columns)
                    {
                        dt.Columns.Add(columnName);
                    }

                    while ((line = sr.ReadLine()) != null)
                    {
                        DataRow dr = dt.NewRow();
                        string[] dataColumns = line.Split('\t');
                        dt.Rows.Add(dataColumns);
                    }
                    return dt;
                }
                throw new Exception(string.Format("File Content in {0} is empty", csvFilePath));
            }
        }
    }
}
